import os
import pathlib

from fastapi import FastAPI
from fastapi.responses import FileResponse
from pydantic import BaseModel
from fastapi.testclient import TestClient
from AccelBrainBeat.brainbeat.binaural_beat import BinauralBeat
import pydub
import random

from blend_av import blend_av

app = FastAPI()


class App(BaseModel):
    app: str


class AudioProperties(BaseModel):
    """
    Audio properties
    """
    main_frequency: int
    lag_frequency: int
    volume: float
    breath_pattern_in: int
    breath_pattern_out: int
    bell_sound: str


class VideoProperties(BaseModel):
    """
    Video properties
    """
    main_frequency: int
    breath_pattern_in: int
    breath_pattern_out: int
    color_palette = str


class AvProperties(BaseModel):
    """
    Audio and video properties
    """
    main_frequency: int
    lag_frequency: int
    volume: float
    breath_pattern_in: int
    breath_pattern_out: int
    bell_sound: str
    color_palette = str


@app.get("/")
async def get_root():
    appresponse = App(app='breathaudio')
    return appresponse


@app.get("/generate_audio")
async def get_generate_audio(audio_properties: AudioProperties):
    """
    Generate audio
    :param audio_properties:
    :return:
    """
    # Generate a random 32 hexadecimal string
    tmp_dir = pathlib.Path("tmp_sound")
    tmp_dir.mkdir(parents=True, exist_ok=True)
    # Remove existing files in tmp_dir
    for filename in os.listdir(tmp_dir):
        os.remove(str(tmp_dir / filename))
    bell_dir = pathlib.Path("bell")
    filename_prefix = ''.join([random.choice('0123456789ABCDEF') for i in range(32)])
    filename = f"{filename_prefix}.wav"
    # Generate binaural beat
    b = BinauralBeat()
    b.save_beat(output_file_name=str(tmp_dir / "binaural_beat.wav"),
                frequencys=(audio_properties.main_frequency,
                            audio_properties.lag_frequency),
                play_time=(audio_properties.breath_pattern_in + audio_properties.breath_pattern_out)*2,
                volume=audio_properties.volume
                )
    pydub_background = pydub.AudioSegment.from_wav(tmp_dir / "binaural_beat.wav")
    pydub_bell = pydub.AudioSegment.from_wav(bell_dir / audio_properties.bell_sound)
    pydub_mix = pydub_background. \
        overlay(pydub_bell, position=0). \
        overlay(pydub_bell, position=audio_properties.breath_pattern_in * 1000). \
        overlay(pydub_bell, position=(audio_properties.breath_pattern_in+audio_properties.breath_pattern_out) * 1000). \
        overlay(pydub_bell, position=(audio_properties.breath_pattern_in*2+audio_properties.breath_pattern_out) * 1000)
    pydub_mix.export(tmp_dir / filename, format="wav")
    return filename


@app.get("/generate_video")
async def get_generate_video(video_properties: VideoProperties):
    """
    Generate video
    :param video_properties:
    :return:
    """
    return None


@app.get("/generate_av")
async def get_generate_av(av_properties: AvProperties):
    """
    Generate audio and video using blend_av
    :param av_properties:
    :return:
    """

    a = AudioProperties()
    a.main_frequency = av_properties.main_frequency
    a.breath_pattern_in = av_properties.breath_pattern_in
    a.breath_pattern_out = av_properties.breath_pattern_out
    a.bell_sound = av_properties.bell_sound
    audio = get_generate_audio(a)

    v = VideoProperties()
    v.main_frequency = av_properties.main_frequency
    v.breath_pattern_in = av_properties.breath_pattern_in
    v.breath_pattern_out = av_properties.breath_pattern_out
    v.color_palette = av_properties.color_palette
    video = get_generate_video(v)

    av = blend_av(audio, video)
    return FileResponse(av)


client = TestClient(app)


def test_getroot():
    response = client.get("/")
    assert response.status_code == 200
